import sys, struct, hashlib
from cassette import Wave, get_fsk_blocks, get_manchester_blocks, irg_manchester, irg_fsk, zero_crossing, sum8

systems=[
{'name': 'Coelsa Injektor', 
 'ver': 'x', 
 'must_have_fsk': [(0, 132, '89c178f675551351518d19190ff1a357')], 
 'must_have_manchester': [
   (0, 132, 'c41d26b38f049dbed4f74ea5437cb05e'), 
   (1, 505, 'e5ec21ecb8f586b3efd3af34625412ea'), 
   (2, 1268, '2b2cd67d400b2818b150cfdd4d0b2d39')], 
 'block_start': 3, 
 'skip_bytes': 1},
{'name': 'Cain Injektor', 
 'ver': 'x', 
 'must_have_fsk': [(0, 132, '0d01315726c71d8cbab1b5905bc4857f')], 
 'must_have_manchester': [
   (0, 723, '50b74c24f1cadb39480da3d3e228d96a'), 
   (1, 26, None)], 
 'block_start': 2, 
 'skip_bytes': 3}]

def check_blocks(must_have, blocks):
  for must in must_have:
    block=blocks[must[0]]['data']
    if len(block)!=must[1]: return False
    h = hashlib.md5()
    h.update(bytearray(block))
    if must[2]!=None and h.hexdigest()!=must[2]: return False
  return True

def system_select(checks):
  for system in systems:
    found=True
    for must_have, blocks in checks.items():
      found=found and check_blocks(system[must_have], blocks)
    if found: return system
  return None

def clean_xex(xex):
  i=0
  while True:
    if xex[i+0]==0xff and xex[i+1]==0xff:
      i+=2
    elif i==0: return None
    start,end=xex[i+1]*256+xex[i+0],xex[i+3]*256+xex[i+2]
    if start>end or (i+end-start+5)>len(xex): return xex[:i]
    i+=4
    sys.stdout.write("segment: %04x-%04x\n" %  (start, end))
    i+=end-start+1


def main():
  sys.stdout.write("injektor2xex 0.2\n")

  if len(sys.argv)<3:
    sys.stderr.write("usage: %s <input.wav> <output.xex>" % sys.argv[0])
    return

  w=Wave(sys.argv[1])

  fsk_blocks=get_fsk_blocks(w, min_irg=200000)
  manchester_blocks=get_manchester_blocks(w, min_irg=900)
  
  sys.stdout.write("%d FSK blocks found\n" % len(fsk_blocks))
  sys.stdout.write("%d manchester blocks found\n" % len(manchester_blocks))

  for block in fsk_blocks+manchester_blocks:
    if block['checksum']==False:
      sys.stderr.write("Bad checksum at block starting at sample %d.\n" % block['irg_end'])
      return

  if len(fsk_blocks)==0 or len(manchester_blocks)==0:
    sys.stderr.write("FSK nor manchester block number can't be zero.\n")
    return   

  system=system_select({'must_have_fsk': fsk_blocks, 'must_have_manchester': manchester_blocks})

  if system==None:
    for b in manchester_blocks:
      h = hashlib.md5()
      h.update(bytearray(b['data']))
      sys.stderr.write("%s %d %r\n" % (h.hexdigest(), len(b['data']), b['checksum']))
    sys.stderr.write("Can't detect system.\n")
  else:
    sys.stdout.write("System %s version %s found.\n" % (system['name'], system['ver']))
    block_start=system['block_start']
    skip_bytes=system['skip_bytes']

    xex=[]
    for b in manchester_blocks[block_start:]:
      data=b['data']
      if data[skip_bytes-1]!=254:
        xex+=data[skip_bytes:-1]

    xex=clean_xex(xex)
    if xex==None:
      sys.stderr.write("Invalid executable.\n")
    else:
      sys.stdout.write("Saving executable %s\n" % sys.argv[2])
      f=open(sys.argv[2], "wb")
      f.write(struct.pack("%dB" % len(xex), *xex))
      f.close()

main()
